/*---------------------------------------------------------------------------

    Filename: options_form.js
    Project:  Lokad Forecaster
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:         Form that exposes the functionality of the Lokad Forecaster
    Developer(s):    Jeff Engel

---------------------------------------------------------------------------*/


class OptionsForm extends Form
{
    // public events
    var forecastClick;


    function OptionsForm()
    {
        super("Lokad Time-Series Forecaster");
        setClientSize(432, 430);
        setMinSize(this.getSize());
        setMaxSize(800, this.getSize().height);
        m_db = new DbConnection;
        
        forecastClick = new Event;


        // source layout

        m_sourcetableTextbox = new TextBox();
        m_sourcetableTextbox.enterPressed.connect(this, sourcePathEnterPressed);

        var browseTablesButton = new Button("Browse...");
        browseTablesButton.click.connect(this, sourcePathBrowseClicked);
        var sourcetableLayout = new BoxLayout(Layout.Horizontal);
        sourcetableLayout.add(new Label("Table Path:", 0, 0, 85), 0, Layout.Center);
        sourcetableLayout.add(m_sourcetableTextbox, 1, Layout.Expand);
        sourcetableLayout.addSpacer(8);
        sourcetableLayout.add(browseTablesButton);

        m_timecolumnChoicebox = new ChoiceBox();
        m_timecolumnChoicebox.selectionChanged.connect(this, timeColumnChanged);
        var timecolumnLayout = new BoxLayout(Layout.Horizontal);
        timecolumnLayout.add(new Label("Time:", 0, 0, 85), 0, Layout.Center);
        timecolumnLayout.add(m_timecolumnChoicebox, 1, Layout.Expand);

        m_seriescolumnChoicebox = new ChoiceBox();
        m_seriescolumnChoicebox.selectionChanged.connect(this, timeColumnChanged);
        var seriescolumnLayout = new BoxLayout(Layout.Horizontal);
        seriescolumnLayout.add(new Label("Values:", 0, 0, 85), 0, Layout.Center);
        seriescolumnLayout.add(m_seriescolumnChoicebox, 1, Layout.Expand);

        m_serieidcolumnChoicebox = new ChoiceBox();
        m_serieidcolumnChoicebox.selectionChanged.connect(this, timeColumnChanged);
        var serieidcolumnLayout = new BoxLayout(Layout.Horizontal);
        serieidcolumnLayout.add(new Label("ID (optional):", 0, 0, 85), 0, Layout.Center);
        serieidcolumnLayout.add(m_serieidcolumnChoicebox, 1, Layout.Expand);


        var sourceLayout = new BorderBoxLayout(Layout.Vertical, "Source Table");
        sourceLayout.addSpacer(4);
        sourceLayout.add(sourcetableLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 10);
        sourceLayout.addSpacer(8);
        sourceLayout.add(timecolumnLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 10);
        sourceLayout.addSpacer(8);
        sourceLayout.add(seriescolumnLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 10);
        sourceLayout.addSpacer(8);
        sourceLayout.add(serieidcolumnLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 10);
        sourceLayout.addSpacer(10);


        // destination layout

        m_destinationtableTextbox = new TextBox();
        var browseDestinationTablesButton = new Button("Browse...");
        browseDestinationTablesButton.click.connect(this, destinationPathBrowseClicked);
        var destinationtableLayout = new BoxLayout(Layout.Horizontal);
        destinationtableLayout.add(new Label("Table Path:", 0, 0, 85), 0, Layout.Center);
        destinationtableLayout.add(m_destinationtableTextbox, 1, Layout.Expand);
        destinationtableLayout.addSpacer(8);
        destinationtableLayout.add(browseDestinationTablesButton);

        m_intervalChoicebox = new ChoiceBox();
        m_intervalChoicebox.add("Quarter Hour");
        m_intervalChoicebox.add("Half Hour");
        m_intervalChoicebox.add("Hour");
        m_intervalChoicebox.add("Day");
        m_intervalChoicebox.add("Week");
        m_intervalChoicebox.add("Month");
        m_intervalChoicebox.add("Quarter");
        m_intervalChoicebox.add("Semester");
        m_intervalChoicebox.add("Year");
        m_intervalChoicebox.selectionChanged.connect(this, intervalSelectionChanged);
        var intervalLayout = new BoxLayout(Layout.Horizontal);
        intervalLayout.add(new Label("Interval:", 0, 0, 85), 0, Layout.Center);
        intervalLayout.add(m_intervalChoicebox, 1, Layout.Expand);

        m_futurerecordsSpinbox = new SpinBox();
        m_futurerecordsLabel = new Label("(Max: 0000)");
        var futurerecordsLayout = new BoxLayout(Layout.Horizontal);
        futurerecordsLayout.add(new Label("Future Records:", 0, 0, 85), 0, Layout.Center);
        futurerecordsLayout.add(m_futurerecordsSpinbox, 1, Layout.Expand);
        futurerecordsLayout.addSpacer(8);
        futurerecordsLayout.add(m_futurerecordsLabel, 0, Layout.Center);

        m_openoncecompletedCheckbox = new CheckBox("Open table once complete");
        var openoncecompletedLayout = new BoxLayout(Layout.Horizontal);
        openoncecompletedLayout.addSpacer(85);
        openoncecompletedLayout.add(m_openoncecompletedCheckbox, 0, Layout.Expand);
        
        m_delaySpinbox = new SpinBox(0, 0, 50);
        m_delaySpinbox.setRange(0, 600);
        m_delaySpinbox.setValue(60);
        m_delayLabel = new Label(" minutes");
        m_delaySpinbox.changed.connect(this, delaySpinboxChanged);
        var delayLayout = new BoxLayout(Layout.Horizontal);
        delayLayout.add(new Label("Processing Time:", 0, 0, 85), 0, Layout.Center);
        delayLayout.add(m_delaySpinbox);
        delayLayout.add(m_delayLabel, 0, Layout.Center);
        delayLayout.addStretchSpacer(1);


        var destinationLayout = new BorderBoxLayout(Layout.Vertical, "Output Table");
        destinationLayout.addSpacer(4);
        destinationLayout.add(destinationtableLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 10);
        destinationLayout.addSpacer(8);
        destinationLayout.add(intervalLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 10);
        destinationLayout.addSpacer(8);
        destinationLayout.add(futurerecordsLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 10);
        destinationLayout.addSpacer(8);
        destinationLayout.add(delayLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 10);
        destinationLayout.addSpacer(8);
        destinationLayout.add(openoncecompletedLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 10);
        destinationLayout.addSpacer(10);


        // bottom layout

        var helpButton = new Button("Help...");
        helpButton.click.connect(this, helpClicked);
        
        var pref = new HostPreferences;
        if (pref.getValue("extensions.lokadforecaster.password"))
        {
            m_resetloginButton = new Button("Reset Login");
            m_resetloginButton.click.connect(this, resetloginClicked);
        }

        var okButton = new Button("Get Forecast");
        okButton.click.connect(this, okClicked);

        var cancelButton = new Button("Cancel");
        cancelButton.click.connect(this, cancelClicked);

        m_buttonsLayout = new BoxLayout(Layout.Horizontal);
        m_buttonsLayout.add(helpButton);
        m_buttonsLayout.addSpacer(8);
        if (m_resetloginButton)
            m_buttonsLayout.add(m_resetloginButton);
        m_buttonsLayout.addStretchSpacer(1);
        m_buttonsLayout.add(okButton);
        m_buttonsLayout.addSpacer(8);
        m_buttonsLayout.add(cancelButton);


        var mainLayout = new BoxLayout(Layout.Vertical);
        mainLayout.addSpacer(8);
        mainLayout.add(sourceLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 8);
        mainLayout.addSpacer(8);
        mainLayout.add(destinationLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 8);
        mainLayout.addSpacer(12);
        mainLayout.add(m_buttonsLayout, 0,
                    Layout.Expand | Layout.Left | Layout.Right, 8);
        mainLayout.addSpacer(8);

        this.setLayout(mainLayout);
    }


    // --- public accessors & mutator ---
    
    function getSourceTable()
    {
        return m_sourcetableTextbox.getText();
    }
    
    function getTimeColumn()
    {
        return m_timecolumnChoicebox.getString();
    }
    
    function getSeriesColumn()
    {
        return m_seriescolumnChoicebox.getString();
    }
    
    function getSerieIdColumn()
    {
        return m_serieidcolumnChoicebox.getSelection() == 0
                                        ? undefined
                                        : m_serieidcolumnChoicebox.getString();
    }
    
    function getDestinationTable()
    {
        return m_destinationtableTextbox.getText();
    }
    
    function getInterval()
    {
        return m_intervalChoicebox.getString();
    }
    
    function getFutureRecords()
    {
        return m_futurerecordsSpinbox.getValue();
    }
    
    function getOpenOnceCompleted()
    {
        return m_openoncecompletedCheckbox.getValue();
    }

    function getDelay()
    {
        return m_delaySpinbox.getValue();
    }
    
    
    function setSourceTable(sourcepath)
    {
        if (m_db.describeTable(sourcepath) != null)
        {
            m_sourcetableTextbox.setText(sourcepath);
            return sourcePathEnterPressed();
        }
        else
            return null;
    }


    // --- private event handlers ---
    
    function sourcePathBrowseClicked()
    {
        var dialog = new ProjectFileDialog(ProjectFileDialog.OpenFile);
        dialog.setCaption("Select a time-series table");

        if (dialog.showDialog() == true)
        {
            m_sourcetableTextbox.setText(dialog.getPath());
            sourcePathEnterPressed();
        }
    }
    
    function sourcePathEnterPressed()
    {
        m_timecolumnChoicebox.clear();
        m_seriescolumnChoicebox.clear();
        m_serieidcolumnChoicebox.clear();

        var sourcepath = m_sourcetableTextbox.getText();
        var fields = m_db.describeTable(sourcepath);

        if (!fields)
            return false;

        this.disablePaint();
        
        m_serieidcolumnChoicebox.add("---");
        m_serieidcolumnChoicebox.select(0);

        for (var i in fields)
        {
            if (fields[i].type == DbType.Date
                    || fields[i].type == DbType.DateTime)
            {
                m_timecolumnChoicebox.add(fields[i].name);
            }
            else if (fields[i].type == DbType.Numeric
                    || fields[i].type == DbType.Integer
                    || fields[i].type == DbType.Double)
            {
                m_seriescolumnChoicebox.add(fields[i].name);
            }
            
            m_serieidcolumnChoicebox.add(fields[i].name);

            if (fields[i].type == DbType.Character
                    && m_serieidcolumnChoicebox.getSelection() == 0)
            {
                m_serieidcolumnChoicebox.select(i + 1);
            }
        }
        
        m_timecolumnChoicebox.select(0);
        m_seriescolumnChoicebox.select(0);

        timeColumnChanged();

        m_destinationtableTextbox.setText(sourcepath + "_Forecast");


        this.enablePaint();

        return true;
    }
    
    function timeColumnChanged()
    {
        var timeCol = m_timecolumnChoicebox.getString();
        if(timeCol == "")
            return;

        var result = m_db.execute("SELECT " + timeCol + " FROM "
            + m_sourcetableTextbox.getText());

        if(result && result.hasNext() && result.next() && result.hasNext())
        {
            var d1 = new Date(result.getValue(0));
            result.next();
            var d2 = new Date(result.getValue(0));

            var difference = Math.abs(d2.getTime() - d1.getTime()) / 60000;
            var lookup = {15: 0, 30: 1, 60: 2, 1440: 3, 10080: 4,
                            40320: 5, 41760: 5, 43200: 5, 44640: 5,
                            525600: 8, 527040: 8};

            if(lookup[difference] !== undefined)
            {
                m_intervalChoicebox.select(lookup[difference]);
                intervalSelectionChanged();
            }
        }
        
    }
    
    function intervalSelectionChanged()
    {
        var interval = m_intervalChoicebox.getString();
        var max = 64;

        if(interval == "Hour")
            max = 336;
        else if(interval == "Half Hour")
            max = 672;
        else if(interval == "Quarter Hour")
            max = 1344;

        m_futurerecordsSpinbox.setValue(max);
        m_futurerecordsLabel.setLabel("(Max: " + max + ")");
        m_futurerecordsSpinbox.setRange(1, max);
    }

    function destinationPathBrowseClicked()
    {
        var dialog = new ProjectFileDialog(ProjectFileDialog.SaveFile);
        dialog.setCaption("Select a table destination");

        if (dialog.showDialog() == true)
        {
            m_destinationtableTextbox.setText(dialog.getPath());
        }
    }

    function delaySpinboxChanged()
    {
        m_delayLabel.setLabel(" minute"
                            + (m_delaySpinbox.getValue() == 1 ? "" : "s"));
    }

    function helpClicked()
    {
        HostApp.openWeb("http://www.kirix.com/extensions/lokad-forecasting-plug-in/");
    }
    
    function resetloginClicked()
    {
        var pref = new HostPreferences;
        pref.deleteValue("extensions.lokadforecaster.username");
        pref.deleteValue("extensions.lokadforecaster.password");
        m_buttonsLayout.show(m_resetloginButton, false);
        m_buttonsLayout.layout();
    }

    function cancelClicked()
    {
        this.show(false);
    }
    
    function okClicked()
    {
        try
        {
            if (m_db.describeTable(m_sourcetableTextbox.getText()) == null)
                throw "Error: Invalid source table.\n\nPlease select a valid table"
                    + " to run the time-series forecast against.";


            if (m_timecolumnChoicebox.getSelection() == -1)
                throw "Error: Invalid Time column.\n\nPlease verify that a column "
                    + "is selected and that it's datatype is either date or datetime.";


            if (m_seriescolumnChoicebox.getSelection() == -1)
                throw "Error: Invalid Series column.\n\nPlease verify that a column "
                    + "is selected and that it's datatype is numeric.";


            if (m_serieidcolumnChoicebox.getString() == m_timecolumnChoicebox.getString())
                throw "Error: Invalid Series ID column.\n\nPlease verify that your "
                    +"series ID column is different than the Time column.";


            if (m_serieidcolumnChoicebox.getString() == m_seriescolumnChoicebox.getString())
                throw "Error: Invalid Series ID column.\n\nPlease verify that your "
                    +"series ID column is different than the Series column.";

            
            if (!m_destinationtableTextbox.getText().length)
                throw "Error: Invalid destination path.\n\nPlease select a path "
                    + "where the forecast results can be stored.";


            if (m_intervalChoicebox.getSelection() == -1)
                throw "Error: Invalid forecast interval.\n\nPlease select a valid "
                    + "interval from the list of choices.";
        }
        catch(e)
        {
            alert(e);
            return;
        }

        this.show(false);
        forecastClick.fire(this);
    }


    // private member variables
    var m_sourcetableTextbox, m_timecolumnChoicebox, m_seriescolumnChoicebox,
        m_serieidcolumnChoicebox, m_destinationtableTextbox, 
        m_intervalChoicebox, m_pastrecordsLabel, m_futurerecordsSpinbox,
        m_futurerecordsLabel, m_openoncecompletedCheckbox, m_delaySpinbox,
        m_delayLabel, m_buttonsLayout, m_resetloginButton;
    var m_db;
}
