
/*---------------------------------------------------------------------------

    Filename: form_logo_login.js
    Project:  Extension Template
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:         General form for showing a company or organization
                       logo and inputting a username and password;
                     Modifications for added functionality
    Developer(s):    David Z. Williams
                     Jeff Engel

---------------------------------------------------------------------------*/


class LogoLoginForm extends Form
{
    // member variables
    var m_username_textbox;
    var m_password_textbox;
    var m_remember_checkbox;
    var m_createaccount_button;
    var m_ok_button;
    var m_cancel_button;
    
    // event variables
    var createaccount_click;
    var ok_click;
    var cancel_click;
    
    function LogoLoginForm()
    {
        // set form size and caption
        super("Login", 100, 100, 450, 300);
        
        // make sure the form can't be resized
        setMinSize(450, 300);
        setMaxSize(450, 300);
        
        // create buttons
        m_createaccount_button = new Button("Get a Lokad account...");
        m_ok_button = new Button("OK");
        m_cancel_button = new Button("Cancel");
        
        // connect button events
        m_createaccount_button.click.connect(this, onCreateAccountClicked);
        m_ok_button.click.connect(this, onOkClicked);
        m_cancel_button.click.connect(this, onCancelClicked);
        
        // create labels
        var username_label = new Label("Username:", 0, 0, 65);
        var password_label = new Label("Password:", 0, 0, 65);
        
        // create textbox controls
        m_username_textbox = new TextBox("", 0, 0, 200);
        m_password_textbox = new TextBox("", 0, 0, 200);
        m_password_textbox.setPasswordMode();

        // create checkbox control
        m_remember_checkbox = new CheckBox("Remember me next time.",
                                           0, 0, 200);

        // add a logo to the login form
        var bmp = new Bitmap;
        var bmp = Extension.getBitmapResource("logo.png");
        var logo_picturebox = new PictureBox(0,
                                             0,
                                             bmp.getWidth(),
                                             bmp.getHeight());
        logo_picturebox.setImage(bmp);
        
        // create username layout
        var username_layout = new BoxLayout(Layout.Horizontal);
        username_layout.addStretchSpacer();
        username_layout.add(username_label, 0, Layout.Center);
        username_layout.add(m_username_textbox, 0, Layout.Center);
        username_layout.addStretchSpacer();

        // create password layout
        var password_layout = new BoxLayout(Layout.Horizontal);
        password_layout.addStretchSpacer();
        password_layout.add(password_label, 0, Layout.Center);
        password_layout.add(m_password_textbox, 0, Layout.Center);
        password_layout.addStretchSpacer();
        
        // create remember layout
        var remember_layout = new BoxLayout(Layout.Horizontal);
        remember_layout.addStretchSpacer();
        remember_layout.addSpacer(65);
        remember_layout.add(m_remember_checkbox, 0, Layout.Center);
        remember_layout.addStretchSpacer();
        
        // create button layout
        var button_layout = new BoxLayout(Layout.Horizontal);
        button_layout.add(m_createaccount_button, 0, Layout.Center);
        button_layout.addStretchSpacer();
        button_layout.add(m_ok_button, 0, Layout.Center);
        button_layout.addSpacer(8);
        button_layout.add(m_cancel_button, 0, Layout.Center);
        
        // create main layout
        var main_layout = new BoxLayout(Layout.Vertical);
        main_layout.addStretchSpacer();
        main_layout.add(logo_picturebox, 0, Layout.Center);
        main_layout.addStretchSpacer();
        main_layout.add(username_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 8);
        main_layout.addSpacer(8);
        main_layout.add(password_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 8);
        main_layout.addSpacer(8);
        main_layout.add(remember_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 8);
        main_layout.addStretchSpacer();
        main_layout.add(button_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 8);
        main_layout.addSpacer(8);
        setLayout(main_layout);
        
        // set the cursor to start in the username textbox
        m_username_textbox.setFocus();
        
        // create button events
        createaccount_click = new Event;
        ok_click = new Event;
        cancel_click = new Event;
    }
    
    function getUsername()
    {
        return m_username_textbox.getText();
    }
    
    function getPassword()
    {
        return m_password_textbox.getText();
    }
    
    function onCreateAccountClicked()
    {
        createaccount_click.fire(this);
    }
    
    function onOkClicked()
    {
        // fire an event letting the caller
        // know we've clicked the ok button
        var args = new Array;
        args.push(m_username_textbox.getText());
        args.push(m_password_textbox.getText());
        args.push(m_remember_checkbox.getValue());
        ok_click.fire(this, args);
    }
    
    function onCancelClicked()
    {
        // fire an event letting the caller
        // know we've clicked the cancel button
        cancel_click.fire(this);
    }
}
